# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=C0103,E0201,E0213,W0232
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Package svnmailer_test.unit.settings

Testcases for svnmailer.settings

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

from svnmailer_test import base
_tests = []


#### Test Bases                                     ####
########################################################
class TestBase(base.TestBase):
    """ Base class for settings tests """
    LOAD_MODULES = (('_settings', 'svnmailer.settings'),)

    def getInitialMocks(self):
        """ Returns the initial mocks """
        return [
            self._mock_svnmailer_util(),
        ]


#### Test Cases                                     ####
########################################################
class _init_ErrorTestCase(TestBase):
    """ svnmailer.settings.Error """

    def test00general(self):
        """ Error is a public Exception """
        expected = Exception
        result = self._settings.Error

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._settings),
            "Is not public")

_tests.append(_init_ErrorTestCase)


class _init_TokensTestCase(TestBase):
    """ svnmailer.settings.Tokens """
    PREFIX = "Tokens"

    def test00general(self):
        """ Token is a public object """
        expected = object
        result = self._settings.Tokens

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._settings),
            "Is not public")

    def test01initial(self):
        """ .valid_tokens is initially empty """
        self.mock.result = self._settings.Tokens.valid_tokens
        self.expect()

    def test02init(self):
        """ .__init__ set initializes valid_tokens / tokens as attributes """
        tok = self._settings.Tokens('a', 'b', 'c')
        self.mock.result = [tok.valid_tokens, tok.a, tok.b, tok.c]

        self.expect(('a', 'b', 'c'), 'a', 'b', 'c')

    def test03init(self):
        """ .__init__ encodes attribute names to ascii """
        tok = self._settings.Tokens(u'a', u'b', u'c')
        self.mock.result = [tok.valid_tokens, tok.a, tok.b, tok.c]

        self.failUnless(isinstance(tok.valid_tokens[0], unicode), "No unicode")
        self.failUnless(isinstance(tok.a, unicode), "No unicode")
        self.expect((u'a', u'b', u'c'), u'a', u'b', u'c')

    def test04init(self):
        """ .__init__ raises UnicodeError """
        self.failUnlessRaises(UnicodeError, self._settings.Tokens, u'\xe9')

    def test05repr(self):
        """ .__repr__ returns proper representation """
        class MyTokens(self._settings.Tokens):
            """ just for the name """
            pass
        inst = MyTokens('foo', u'bar')
        self.mock.result.append(repr(inst))

        self.expect("MyTokens('foo', u'bar')")

_tests.append(_init_TokensTestCase)


class _init_ModesTestCase(TestBase):
    """ svnmailer.settings.MODES """

    def test00general(self):
        """ MODES is a public Tokens instance """
        expected = self._settings.Tokens
        result = self._settings.MODES

        self.failUnless(isinstance(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless('MODES' in self.public(self._settings),
            "Is not public")

    def test01initial(self):
        """ MODES is initially filled properly """
        self.mock.result = self._settings.MODES.valid_tokens

        self.failUnless(isinstance(self.mock.result[0], str), "No str")
        self.expect('commit', 'propchange', 'lock', 'unlock')

_tests.append(_init_ModesTestCase)


class _init_XpathTestCase(TestBase):
    """ svnmailer.settings.XPATH """

    def test00general(self):
        """ XPATH is a public Tokens instance """
        expected = self._settings.Tokens
        result = self._settings.XPATH

        self.failUnless(isinstance(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless('XPATH' in self.public(self._settings),
            "Is not public")

    def test01initial(self):
        """ XPATH is initially filled properly """
        self.mock.result = self._settings.XPATH.valid_tokens

        self.failUnless(isinstance(self.mock.result[0], unicode), "No unicode")
        self.expect(u'yes', u'no', u'ignore')

_tests.append(_init_XpathTestCase)


class _init_ShowencTestcase(TestBase):
    """ svnmailer.settings.SHOWENC """

    def test00general(self):
        """ SHOWENC is a public Tokens instance """
        expected = self._settings.Tokens
        result = self._settings.SHOWENC

        self.failUnless(isinstance(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless('SHOWENC' in self.public(self._settings),
            "Is not public")

    def test01initial(self):
        """ SHOWENC is initially filled properly """
        self.mock.result = self._settings.SHOWENC.valid_tokens

        self.failUnless(isinstance(self.mock.result[0], unicode), "No unicode")
        self.expect(u'yes', u'no', u'nondefault')

_tests.append(_init_ShowencTestcase)


class _init_ManagerTestCase(TestBase):
    """ svnmailer.settings.Manager """
    PREFIX = "Manager"

    def test00general(self):
        """ Manager is a public util.Singleton """
        expected = self.mock['svnmailer.util'].Singleton
        result = self._settings.Manager

        self.failUnless(issubclass(result, expected),
            "Is not an %r, but %r" % (expected, result))
        self.failUnless(result.__name__ in self.public(self._settings),
            "Is not public")

    def test01initial(self):
        """ ._LOADER is properly filled initially """
        self.mock.result = [self._settings.Manager._LOADER]

        self.expect('svnmailer.settings.configfile.ConfigFileSettings')

    def test02initial(self):
        """ ._TYPEMAP is properly filled initially """
        self.mock.result = [self._settings.Manager._TYPEMAP]

        self.expect({
            'unicode'   : 'svnmailer.settings._accessors.UnicodeMember',
            'string'    : 'svnmailer.settings._accessors.StringMember',
            'int'       : 'svnmailer.settings._accessors.IntegerMember',
            'bool'      : 'svnmailer.settings._accessors.BooleanMember',
            'humanbool' : 'svnmailer.settings._accessors.HumanBooleanMember',
            'regex'     : 'svnmailer.settings._accessors.RegexMember',
            'token'     : 'svnmailer.settings._accessors.TokenMember',
            'tokenlist' : 'svnmailer.settings._accessors.TokenlistMember',
            'filename'  : 'svnmailer.settings._accessors.FilenameMember',
            'unicommand': 'svnmailer.settings._accessors.CommandlineMember',
            'quotedstr' : 'svnmailer.settings._accessors.QuotedstringMember',
            'stdin'     : 'svnmailer.settings._accessors.StdinMember',
            'mailaction': 'svnmailer.settings._accessors.MailactionMember',
        })

    def test03initial(self):
        """ ._MEMBERS is properly filled initially """
        self.mock.result = [self._settings.Manager._MEMBERS]

        self.expect({
            'general': {
                'members': {
                    'sendmail_command'  : ('unicommand', {'map': True}),
                    'smtp_host'         : ('unicode',    {'map': True}),
                    'smtp_user'         : ('quotedstr',  {'map': True}),
                    'smtp_pass'         : ('quotedstr',  {'map': True}),
                    'nntp_host'         : ('unicode',    {'map': True}),
                    'nntp_user'         : ('quotedstr',  {'map': True}),
                    'nntp_pass'         : ('quotedstr',  {'map': True}),
                    'debug_all_mails_to': ('tokenlist',  {'map': True}),
                    'cia_rpc_server'    : ('unicode',    {'map': True}),
                    'tempdir'           : ('filename',   {'map': True}),
                    'diff_command'      : 'unicommand',
                },
                'aliases': {
                    'mail_command' : 'sendmail_command',
                    'smtp_hostname': 'smtp_host',
                    'diff'         : 'diff_command',
                },
            },

            'group'  : {
                'members': {
                    '_name'                      : 'unicode',
                    '_def_for_repos'             : 'regex',
                    '_def_for_paths'             : 'regex',
                    'for_repos'                  : ('regex', {'map': True}),
                    'for_paths'                  : ('regex', {'map': True}),
                    'exclude_paths'              : ('regex', {'map': True}),
                    'ignore_if_other_matches'    : 'humanbool',
                    'show_nonmatching_paths'     : ('token', {'map': True,
                                       'allowed': (u'yes', u'no', u'ignore')}),
                    'commit_subject_template'    : ('unicode', {'map': True}),
                    'propchange_subject_template': ('unicode', {'map': True}),
                    'lock_subject_template'      : ('unicode', {'map': True}),
                    'unlock_subject_template'    : ('unicode', {'map': True}),
                    'commit_subject_prefix'      : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'propchange_subject_prefix'  : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'lock_subject_prefix'        : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'unlock_subject_prefix'      : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'max_subject_length'         : 'int',
                    'from_addr'                  : ('tokenlist',
                                                 {'subst': True, 'map': True}),
                    'to_addr'                    : ('tokenlist',
                                                 {'subst': True, 'map': True}),
                    'reply_to_addr'              : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'to_newsgroup'               : ('tokenlist',
                                                 {'subst': True, 'map': True}),
                    'diff_command'               : ('unicommand',
                                                 {'map': True}),
                    'generate_diffs'             : 'tokenlist',
                    'browser_base_url'           : ('unicode', {'map': True}),
                    'revision_url'               : ('unicode', {'map': True}),
                    'diff_add_url'               : ('unicode', {'map': True}),
                    'diff_copy_url'              : ('unicode', {'map': True}),
                    'diff_delete_url'            : ('unicode', {'map': True}),
                    'diff_modify_url'            : ('unicode', {'map': True}),
                    'long_mail_action'           : ('mailaction',
                                                 {'map': True}),
                    'long_news_action'           : ('mailaction',
                                                 {'map': True}),
                    'mail_type'                  : ('unicode', {'map': True}),
                    'mail_transfer_encoding'     : 'unicode',
                    'news_transfer_encoding'     : 'unicode',
                    'custom_header'              : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'extract_x509_author'        : 'humanbool',
                    'cia_project_name'           : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'cia_project_module'         : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'cia_project_branch'         : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'cia_project_submodule'      : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'cia_project_path'           : ('unicode',
                                                 {'subst': True, 'map': True}),
                    'apply_charset_property'     : 'humanbool',
                    'show_applied_charset'       : ('token',
                                  {'allowed': (u'yes', u'no', u'nondefault')}),
                    'viewcvs_base_url'           : ('unicode', {'map': True}),
                },
                'aliases': {
                    'suppress_if_match'  : 'ignore_if_other_matches',
                    'fallback'           : 'ignore_if_other_matches',
                    'reply_to'           : 'reply_to_addr',
                    'x509_author'        : 'extract_x509_author',
                    'charset_property'   : 'apply_charset_property',
                    'truncate_subject'   : 'max_subject_length',
                    'subject_length'     : 'max_subject_length',
                    'diff'               : 'diff_command',
                    'nonmatching_paths'  : 'show_nonmatching_paths',
                    'nongroup_paths'     : 'show_nonmatching_paths',
                    'show_nongroup_paths': 'show_nonmatching_paths',
                },
                'eqignore': [
                    '_name',
                    '_def_for_repos', '_def_for_paths',
                    'for_repos', 'for_paths', 'exclude_paths',
                    'ignore_if_other_matches',
                    'to_addr', 'from_addr', 'to_newsgroup',
                    'custom_header',
                    'cia_project_name', 'cia_project_module',
                    'cia_project_branch', 'cia_project_submodule',
                    'cia_project_path',
                ],
            },

            'runtime': {
                'members': {
                    '_repos'       : None,
                    'stdin'        : 'stdin',
                    'path_encoding': 'string',
                    'debug'        : 'bool',
                    'revision'     : 'int',
                    'repository'   : 'filename',
                    'config'       : 'filename',
                    'mode'         : 'string',
                    'author'       : 'unicode',
                    'propname'     : 'unicode',
                    'action'       : 'unicode',  # >= svn 1.2
                },
                'aliases': {},
            },
        })

    def test04initial(self):
        """ ._MAPPERS is properly filled initially """
        self.mock.result = [self._settings.Manager._MAPPERS]

        self.expect(['svnmailer.settings.mappers.PlainMapper'])

    def test05registerType(self):
        """ .registerType registers a new type """
        manager = self._settings.Manager()
        manager.registerType('some:Type', 'foo.class')
        manager = self._settings.Manager()
        self.mock.result = [manager._TYPEMAP['some:Type']]

        self.expect('foo.class')

    def test06registerLoader(self):
        """ .registerLoader registers a new loader """
        manager = self._settings.Manager()
        manager.registerLoader('bar.class')
        manager = self._settings.Manager()
        self.mock.result = [manager._LOADER]

        self.expect('bar.class')

    def test07registerOption(self):
        """ .registerOption registers a new option """
        manager = self._settings.Manager()
        manager.registerOption('general', 'foo_bar')
        manager = self._settings.Manager()
        self.mock.result = [manager._MEMBERS['general']['members']['foo_bar']]

        self.expect(None)

    def test08registerOption(self):
        """ .registerOption registers a new option with spec """
        manager = self._settings.Manager()
        manager.registerOption('runtime', 'foo', spec = 'somespec')
        manager = self._settings.Manager()
        self.mock.result = [manager._MEMBERS['runtime']['members']['foo']]

        self.expect('somespec')

    def test09registerOption(self):
        """ .registerOption registers a new option with aliases """
        manager = self._settings.Manager()
        manager.registerOption('group', 'foo', aliases = ('bar', 'baz'))
        manager = self._settings.Manager()
        self.mock.result = [
            manager._MEMBERS['group']['members']['foo'],
            manager._MEMBERS['group']['aliases']['bar'],
            manager._MEMBERS['group']['aliases']['baz'],
        ]

        self.expect(None, 'foo', 'foo')

    def test10registerOption(self):
        """ .registerOption registers a new option with compare flag """
        self._settings.Manager._MEMBERS['group']['eqignore'] = ['bar']
        manager = self._settings.Manager()
        manager.registerOption('group', 'foo', compare = False)
        manager = self._settings.Manager()
        self.mock.result = [manager._MEMBERS['group']['eqignore']]

        self.expect(['bar', 'foo'])

    def test11registerOption(self):
        """ .registerOption raises KeyError """
        manager = self._settings.Manager()

        self.failUnlessRaises(KeyError, manager.registerOption, 'foo', 'bar')

    def test12registerMapper(self):
        """ .registerMapper registers a new mapper """
        self._settings.Manager._MAPPERS = ['foo']
        manager = self._settings.Manager()
        manager.registerMapper('bar')
        manager = self._settings.Manager()
        self.mock.result = manager._MAPPERS

        self.expect('foo', 'bar')

    def test13loadSettings(self):
        """ .loadSettings runs the loader with the proper arguments """
        class Foo(object):
            """ no doc """
            def __init__(this, *args):
                """ no doc """
                if args:
                    options, members, typemap, mappers = args
                    typemap = dict([
                        (key, "%s.%s" % (val.__module__, val.__name__))
                        for key, val in typemap.items()])
                    mappers = ["%s.%s" % (mapper.__module__, mapper.__name__)
                        for mapper in mappers]
                    args = (options, members, typemap, mappers)
                self.mock.result.extend([this.__class__.__name__, args])

        class MyManager(self._settings.Manager):
            """ no doc """
            _MEMBERS = {'foo': 'bar'}
            _TYPEMAP = {'bar': 'gna.baz'}
            _LOADER = 'zonk.loaderclass'
            _MAPPERS = ['some.mapperclass']

            def _load(this, classname):
                """ no doc """
                idx = classname.rfind('.')
                module, classname = classname[:idx], classname[idx+1:]
                return type(
                    classname,
                    (Foo,) + Foo.__bases__,
                    {'__module__': module}
                )

        inst = MyManager()
        inst.loadSettings('*options')

        self.expect(
            'loaderclass', ('*options', {'foo': 'bar'}, {'bar': 'gna.baz'},
                ['some.mapperclass'])
        )

    def test14load(self):
        """ ._load returns the dotted object """
        manager = self._settings.Manager()
        self.mock.result.append(manager._load('some.class')._mocked)

        self.expect(
            'util.loadDotted', 'some.class',
            'SomeClass'
        )

    def test15load(self):
        """ ._load expects a qualified name """
        manager = self._settings.Manager()

        self.failUnlessRaises(ImportError, manager._load, 'some')

    def test16load(self):
        """ ._load passes ImportError """
        manager = self._settings.Manager()

        self.failUnlessRaises(ImportError, manager._load, 'foo.class')

_tests.append(_init_ManagerTestCase)


class _init_utilTestCase(TestBase):
    """ svnmailer.settings.util """
    
    def test00delete(self):
        """ The util module import is not exposed to the public """
        self.failIf(hasattr(self._settings, 'util'), "util attribute found")

_tests.append(_init_utilTestCase)


class _init_generalMembersTestCase(TestBase):
    """ svnmailer.settings.general_members """
    
    def test00delete(self):
        """ The general_members variable is not exposed to the public """
        self.failIf(hasattr(self._settings, 'general_members'),
            "variable found"
        )

_tests.append(_init_generalMembersTestCase)


class _init_runtimeMembersTestCase(TestBase):
    """ svnmailer.browser.util """
    
    def test00delete(self):
        """ The runtime_members variable is not exposed to the public """
        self.failIf(hasattr(self._settings, 'runtime_members'),
            "variable found"
        )

_tests.append(_init_runtimeMembersTestCase)


class _init_groupMembersTestCase(TestBase):
    """ svnmailer.browser.util """
    
    def test00delete(self):
        """ The group_members variable is not exposed to the public """
        self.failIf(hasattr(self._settings, 'group_members'),
            "variable found"
        )

_tests.append(_init_groupMembersTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
